/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.core;

import java.util.List;

import org.cytoscape.model.CyColumn;
import org.cytoscape.model.CyEdge;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyRow;
import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.cytoUtil.NodeMap;
import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInput;

public class NetworkParser {

	//Symbolic Links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();

	public static LibraryInput parseNetwork (
			CyNetwork net,
			NodeMap nodeMap,
			CyColumn weightColumn,
			CyColumn permanentColumn,
			CyColumn forbiddenColumn,
			double deletionCostDefault
			) throws NetworkParsingException
	{
		//Create an empty instance / fetch C++ object pointer
		LibraryInput generatedInput = new LibraryInput();
		if (net != null){
			//Create an empty full graph with a given size
			generatedInput.setSize(net.getNodeCount());
			//Fetch edges
			List<CyEdge> edges = net.getEdgeList();

			//Loop over edges
			for (CyEdge e : edges) {

				//Parse editing costs
				double weight = deletionCostDefault;

				//Fetch entry and check if it exists
				CyRow edgeEntry = net.getRow(e);

				//Check if there is a weight column defined, else skip
				if (weightColumn != null){
					//Check if the column contains an entry for the respective edge
					//It is possible, that there are missing entries
					if (edgeEntry.get(weightColumn.getName(), weightColumn.getType()) != null){
						if (weightColumn.getType() == Integer.class) {
							weight = 1.0*edgeEntry.get(weightColumn.getName(), Integer.class);
						}
						else if (weightColumn.getType() == Double.class) {
							weight = edgeEntry.get(weightColumn.getName(), Double.class);
						}
					}
				}

				//Parse Forbidden/Permanent markers
				boolean forbidden = false;
				boolean permanent = false;

				if (permanentColumn != null) {
					//Additional check as it is not required to have a value in every row
					if (edgeEntry.get(permanentColumn.getName(), Boolean.class) != null) {
						permanent =  (boolean)edgeEntry.get(permanentColumn.getName(), Boolean.class);
					}
				}
				if (forbiddenColumn != null) {
					//Additional check as it is not required to have a value in every row
					if (edgeEntry.get(forbiddenColumn.getName(), Boolean.class) != null) {
						forbidden =  (boolean)edgeEntry.get(forbiddenColumn.getName(), Boolean.class);
					}
				}

				//Check for edges that are forbidden AND permanent -> Throw exception
				if (forbidden && permanent) {
					throw new NetworkParsingException(LocalizationManager.get("dualInfinityError"));
				}

				logger.debug("Found Edge: "+edgeEntry.get("name", String.class)+ " with weight:"+weight);

				generatedInput.addEdge(
						nodeMap.get(e.getSource()),
						nodeMap.get(e.getTarget()),
						weight,
						permanent,
						forbidden
					);

			}
		}
		return generatedInput;
	}

}
